using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Datory;
using GxPress.Common.AppOptions;
using GxPress.Common.Tools;
using GxPress.Result.Web;
using Microsoft.Extensions.Caching.Distributed;
using Microsoft.Extensions.Options;
using System.Transactions;
using Dapper;
using GxPress.Repository.Interface.Group;

namespace GxPress.Repository.Implement.Group
{
    /// <summary>
    /// 小组广场
    /// </summary>
    public class GroupCategoryRepository : IGroupCategoryRepository
    {
        private readonly Repository<Entity.tede2.Group.GroupCategory> _repository;
        private readonly Repository<Entity.tede2.Group.UserGroupCategory> userGroupCategoryRepository;
        private readonly IMapper _mapper;
        private readonly IDistributedCache _cache;
        private readonly string _connectionString;
        private readonly string _databaseTypestr;
        public GroupCategoryRepository(IOptionsMonitor<DatabaseOptions> dbOptionsAccessor, IMapper mapper, IDistributedCache cache)
        {
            _databaseTypestr = dbOptionsAccessor.CurrentValue.DatabaseType;
            _connectionString = dbOptionsAccessor.CurrentValue.ConnectionString;
            var databaseType = StringUtils.ToEnum<DatabaseType>(dbOptionsAccessor.CurrentValue.DatabaseType, DatabaseType.MySql);
            var database = new Database(databaseType, dbOptionsAccessor.CurrentValue.ConnectionString);
            _mapper = mapper;
            _cache = cache;
            _repository = new Repository<Entity.tede2.Group.GroupCategory>(database);
            userGroupCategoryRepository = new Repository<Entity.tede2.Group.UserGroupCategory>(database);
        }

        public IDatabase Database => _repository.Database;
        public string TableName => _repository.TableName;
        public List<TableColumn> TableColumns => _repository.TableColumns;

        /// <summary>
        /// 添加小组广场
        /// </summary>
        /// <param name="GroupCategory"></param>
        /// <returns></returns>
        public async Task<int> InsertAsync(Entity.tede2.Group.GroupCategory groupCategory)
        {
            groupCategory.ImageUrls = StringUtils.RemoveDomain(groupCategory.ImageUrls);
            var id = await _repository.InsertAsync(groupCategory);
            groupCategory.Id = id;
            groupCategory.Sort = id;
            await UpdateAsync(groupCategory);
            return id;
        }


        public async Task<bool> UpdateAsync(Entity.tede2.Group.GroupCategory groupCategory)
        {
            if (groupCategory.Id > 0)
            {
                var model = await GetAsync(groupCategory.Id);
                if (!string.IsNullOrEmpty(groupCategory.PiazzaName))
                    model.PiazzaName = groupCategory.PiazzaName;
                if (groupCategory.ParentId > 0)
                    model.ParentId = groupCategory.ParentId;
                if (string.IsNullOrEmpty(groupCategory.ImageUrls))
                    model.ImageUrls = StringUtils.RemoveDomain(groupCategory.ImageUrls);
                if (groupCategory.Sort > 0)
                    model.Sort = groupCategory.Sort;
                return await _repository.UpdateAsync(groupCategory);
            }
            return false;
        }

        public async Task<Entity.tede2.Group.GroupCategory> GetAsync(int id)
        {
            return await _repository.GetAsync(id);
        }

        public async Task<bool> DeleteAsync(int id)
        {
            try
            {
                using (var transactionScope = new TransactionScope())
                {
                    await _repository.DeleteAsync(id);
                    transactionScope.Complete();
                }
            }
            catch
            {
                return false;
            }
            return true;
        }
        /// <summary>
        /// 根据parendId获取
        /// </summary>
        /// <param name="parentId"></param>
        /// <returns></returns>
        public async Task<IEnumerable<GroupCategoryResult>> GetAllAsync(int parentId)
        {
            var sql = $@"SELECT 
                            *,
                            (SELECT 
                                    COUNT(1)
                                FROM
                                    tede_group
                                WHERE
                                    GroupCategroyId = a.Id) AS GroupCount
                        FROM
                            tede_group_piazza AS a where a.ParentId={parentId}
                        ORDER BY a.Sort DESC , a.CreatedDate DESC";
            var databaseType = _databaseTypestr.ToEnum<DatabaseType>(DatabaseType.MySql);
            var database = new Database(databaseType, _connectionString);
            var connection = database.GetConnection();
            var result = await connection.QueryAsync<GroupCategoryResult>(sql);
            foreach (var item in result)
            {
                //item.IsChildren = await _repository.ExistsAsync(Q.Where(nameof(Entity.tede2.Group.GroupCategory.ParentId), item.Id));
                item.ImageUrls = StringUtils.AddDomain(item.ImageUrls);
            }
            return result;
        }
        public async Task<IEnumerable<GroupCategoryResult>> GetUsergroupCategoryResult(int userId)
        {
            string sql = $@"SELECT 
                                a.*
                            FROM
                                tede_group_piazza a
                                    INNER JOIN
                                tede_user_group_category b ON a.id = b.GroupCategroyId
                            WHERE
                                b.UserId = {userId}
                            ORDER BY b.CreatedDate";
            var databaseType = _databaseTypestr.ToEnum<DatabaseType>(DatabaseType.MySql);
            var database = new Database(databaseType, _connectionString);
            var connection = database.GetConnection();
            var result = await connection.QueryAsync<GroupCategoryResult>(sql);
            foreach (var item in result)
            {
                item.ImageUrls = StringUtils.AddDomain(item.ImageUrls);
                item.IsChildren = await _repository.ExistsAsync(Q.Where(nameof(Entity.tede2.Group.GroupCategory.ParentId), item.Id));
            }
            return result;
        }

    }
}